Version=1
AutoSorting=1
OnlyRead=0
OnlyOneFolder=1
:Begin ""
!M:11111111.mb\!!!!Outbox\!!!Sent\Sub21_OR_22\
@0:Subject:Subject21	O	IT
@0:Subject:Subject22	O	IT
$O:Sort=1
:End ""
:Begin ""
!M:11111111.mb\!!!!Outbox\!!!Sent\To21_OR_22\
@0:To:To21@example.com	O	I
@0:To:To22@example.com	O	I
$O:Sort=1
:End ""
:Begin ""
!M:11111111.mb\!!!!Outbox\!!!Sent\To21\
@0:To:To21@example.com	O	I
$O:Sort=1
$X:disabled
:End ""
